%examples
% 
% 
%% Example 1 %%
clear all
L=100;   % System length
S=5;    % No-zero values
h=[randn(1,S) zeros(1,L-S)]';
h=h(randperm(L));

N=150; % Number of data (observations)
x=randn(1,N+L-1);
y=conv(h,x);
y=y(L:end-L+1)';

noise_variance=0.1; %noise variance
y=y+randn(N,1)*sqrt(noise_variance);


options = SPALOPTIONS(@APWL1,'Sla',1.3*sqrt(noise_variance),'delta',S,'q',10);
[out]= spal(y,x,L,options,'weighteps',[0.001,1]);
%The previous is equivalent to 
% options = SPALOPTIONS(@APWL1,'Sla',1.3*sqrt(noise_variance),'delta',S,'weighteps',[0.001,1],'q',10);
% [out2]= spal(y,x,L,options);
%and equivalent to
%[out]= spal(y,x,L,@APWL1,'slab',1.3*sqrt(noise_variance),'delta',S,'q',10,'weig',[0.001,1]);


figure
hold on
stem(h)
stem(out,'r--o')


%% Example 2 %%
%similar to Example 1 having Gaussian noise as measurement vectors.
clear all
L=100;   % System length
S=5;    % No-zero values
h=[randn(1,S) zeros(1,L-S)]';
h=h(randperm(L));

N=150; % Number of data (observations)
x=randn(N,L);
y=x*h;

noise_variance=0.1; %noise variance
y=y+randn(N,1)*sqrt(noise_variance);


options = SPALOPTIONS(@APWL1,'Sla',1.3*sqrt(noise_variance),'delta',S,'q',10);
[out]= spal(y,x,L,options,'weighteps',[0.001,1]);

figure
hold on
stem(h)
stem(out,'r--o')


%% Example 3 %%
% Example of a larger signal and error plot
clear all
L=1000;   % System length
S=100;    % No-zero values
h=[randn(1,S) zeros(1,L-S)]';
h=h(randperm(L));

N=2000; % Number of data (observations)

x=randn(N,L);
y=x*h;
% x=randn(1,N+L-1);
% y=conv(h,x);
% y=y(L:end-L+1)';

noise_variance=0.1; %noise variance
y=y+randn(N,1)*sqrt(noise_variance);


[out]= spal(y,x,L,@APWL1,'slab',1.3*sqrt(noise_variance),'delta',S,'q',200,'weig',0.05,'out',2);
er=zeros(1,N);
for i=1:N
    er(i)=sum((h-out(:,i)).^2);
end
figure
plot(1:N,10*log10(er))

% % This is equivalent to the previous 7 lines where the output is evaluated
% % only in the time instances 1:10:N
% xp=1:10:N;
% [out]= spal(y,x,L,@APWL1,'slab',1.3*sqrt(noise_variance),'delta',S,'q',200,'weig',0.05,'out',xp);
% er=zeros(1,length(xp));
% for i=1:length(xp)
%     er(i)=sum((h-out(:,i)).^2);
% end
% figure
% plot(xp,10*log10(er))



%% Example 4 %%
%Finding ensemble averages over a number of independent realizations
clear all
L=200;   % System length
S=30;    % No-zero values
h=[randn(1,S) zeros(1,L-S)]';
h=h(randperm(L));

N=1000; % Number of data (observations)

aver=30;

er1=zeros(1,N);
er2=zeros(1,N);
noise_variance=0.1; %noise variance
opt = SPALOPTIONS(@APWL1,'Sla',1.3*sqrt(noise_variance),'delta',S,'weighteps',[0.001,1],'output',2,'verbose',0);
for rep=1:aver
        if mod(rep,10)==0
        fprintf(' %d',round(rep/aver*100))
        end

        h=[randn(1,S) zeros(1,L-S)]';
h=h(randperm(L));

% x=randn(1,N+L-1);
% y=conv(h,x);
% y=y(L:end-L+1)';
x=randn(N,L);
y=x*h;


y=y+randn(N,1)*sqrt(noise_variance);

 A=repmat(h,1,N);

[out]= spal(y,x,L,opt,'mu',1,'q',30);
er1=er1+sum((A-out).^2,1);

[out]= spal(y,x,L,opt,'mu',[0 2],'q',30);
er2=er2+sum((A-out).^2,1);

end

figure;hold on
h=plot(1:N,log10(er1/aver/L),'b',1:N,log10(er2/aver/L),'r');
legend(h,{'APWL1 with extrapolation', 'APWL1 without extrapolation'})
 
